;;
;;
;; Verifying the digraph package. Code adapted from the Boost graph
;; library dependency example.
;;
;; Version $Revision$
;;
;;
;; Copyright 2007 Ivan Raikov and the Okinawa Institute of Science and Technology
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2 of the
;; License, or (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
;; 02110-1301 USA
;;
;; A full copy of the GPL license can be found on Debian systems in 
;; /usr/share/common-licenses/GPL-2
;;
;;
;;

(require-extension srfi-1)
(require-extension srfi-13)
(require-extension digraph)
(require-extension graph-scc)
(require-extension testbase)
(require-extension testbase-output-compact)


(define used-by
   (list 
     (cons 'dax_h 'foo_cpp) (cons 'dax_h 'bar_cpp) (cons 'dax_h 'yow_h)
     (cons 'yow_h 'bar_cpp) (cons 'yow_h 'zag_cpp) (cons 'boz_h 'bar_cpp)
     (cons 'boz_h 'zig_cpp) (cons 'boz_h 'zag_cpp) (cons 'zow_h 'foo_cpp)
     (cons 'foo_cpp 'foo_o) (cons 'foo_o 'libfoobar_a) 
     (cons 'bar_cpp 'bar_o) (cons 'bar_o 'libfoobar_a) 
     (cons 'libfoobar_a 'libzigzag_a)  (cons 'zig_cpp 'zig_o) 
     (cons 'zig_o 'libzigzag_a) (cons 'libfoobar_a 'dax_h) (cons 'zag_cpp 'zag_o) 
     (cons 'zag_o 'libzigzag_a) (cons 'libzigzag_a 'killerapp)))


(define-expect-unary pair?)


(define-test scc-test "scc test"
  (initial
   (define g (void))
   
   (define node-list
     (delete-duplicates
      (concatenate (list (map car used-by) (map cdr used-by)))))
   
   (define node-ids
     (list-tabulate (length node-list) values))
   
   (define node-map  (zip node-list node-ids)) )


  ;; can we make a graph
  (expect-set! g (make-digraph 'depgraph "dependency graph"))
  
  ;; add the nodes to the graph
  (test-eval 'add-nodes
	     (for-each (lambda (i n) ((g 'add-node!) i n))
		       node-ids node-list))

  ;; make sure all nodes got inserted
  (test/equal 'nodes-inserted?
	      ((g 'nodes))
	      '((14 killerapp)
		(13 libzigzag_a) (12 zag_o) (11 zag_cpp)
		(10 zig_o) (9 zig_cpp) (8 libfoobar_a) (7 bar_o)
		(6 bar_cpp) (5 foo_o) (4 foo_cpp) (3 zow_h) (2 boz_h)
		(1 yow_h) (0 dax_h)))
  
  ;; add the edges to the graph
  (test/collect 'add-edges
		(for-each (lambda (e)
			    (collect-test (expect-pair (conc "valid edge: " e) e))
			    (let* ((ni (car e))
				   (nj (cdr e))
				   (i (car (alist-ref ni node-map)))
				   (j (car (alist-ref nj node-map))))
			      (collect-test
			       (expect-success "adding edge"
					       ((g 'add-edge!) (list i j (format "~A->~A" ni nj))))) ) )
			  used-by))

  (test-eval 'get-roots ((g 'roots)))
  
  (test-let ((roots  ((g 'roots)))
	     (str    (open-output-string)))

	    (test/equal 'graph-scc-fold 
			(graph-scc-fold g (lambda (scc ax) (cons scc ax)) (list))
			'((2) (3) (8 0 1 6 7 4 5) (9) (10) (11) (12) (13) (14))))

  )


(test::styler-set! scc-test test::output-style-compact)
(run-test "scc test")

	   



	   